import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

class OnairosPromptModal extends StatelessWidget {
  final VoidCallback onClose;
  final String logoPath; // Path to the Onairos logo image asset

  OnairosPromptModal({Key? key, required this.onClose, required this.logoPath}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.all(20),
      decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.circular(25),
      ),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Align(
            alignment: Alignment.topRight,
            child: IconButton(
              icon: Icon(Icons.close),
              onPressed: onClose,
            ),
          ),
          SizedBox(height: 20),
          GestureDetector(
            onTap: () async {
              // Attempt to launch the Onairos app, or fallback to the App Store
              var url = Uri.parse('onairos://'); // Use Uri.parse to create the URI
              if (await canLaunchUrl(url)) {
                await launchUrl(url);
              } else {
                var appStoreUrl = Uri.parse('https://apps.apple.com/app/onairos');
                // Replace with the actual URL to the Onairos app in the app store
                await launchUrl(appStoreUrl);
              }
            },
            child: Image.asset(logoPath, width: 100, height: 100),
          ),
          SizedBox(height: 20),
          Text(
            'Please download and login to the Onairos app, then train your model to send data',
            textAlign: TextAlign.center,
            style: TextStyle(fontSize: 16),
          ),
        ],
      ),
    );
  }
}
