import 'dart:convert';
import 'package:uni_links/uni_links.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:onairos/secureAuthService.dart';
import 'package:url_launcher/url_launcher.dart';
import 'overlay.dart';
import 'NoOnairos.dart';


// /Main imported Onaiors button component
class OnairosButtonWrapper1 extends StatefulWidget {
  final Map<String, Map<String, String>> requestData;
  final String returnLink;
  final void Function(String apiUrl, String accessToken) onResolved;
  
  const OnairosButtonWrapper1({
    Key? key,
    required this.requestData,
    required this.returnLink,
    required this.onResolved,
  }) : super(key: key);

  @override
  _OnairosButtonWrapperState createState() => _OnairosButtonWrapperState();
}

class _OnairosButtonWrapperState extends State<OnairosButtonWrapper1> {
  static const MethodChannel _channel = MethodChannel('com.onairos/overlay');
  int nonce = 0;
  @override
  void initState() {
    super.initState();
    initDeepLinkListener();
  }

  // // Place this in your initState or similar initialization method
  // void initDeepLinkListener() {
  //   // Assuming using a package like uni_links or similar for deep linking
  //   uriLinkStream.listen((Uri? uri) {
  //     if (uri != null && uri.queryParameters['callback'] == 'onairosOverlay://callback') {
  //       String username = uri.queryParameters['username'] ?? '';
  //       String encryptedKey = uri.queryParameters['encryptedKey'] ?? '';
  //       handleCallback(username, encryptedKey);
  //     }
  //   }, onError: (err) {
  //     // Handle error scenario
  //   });
  // }


  void initDeepLinkListener() {
    // Replace 'uni_links' package or similar for deep linking
    uriLinkStream.listen((Uri? uri) {
      // Make sure to replace 'app2scheme' with the actual scheme App2 is registered to handle
      if (uri != null && uri.scheme == widget.returnLink && uri.host == 'callback') {
        String username = uri.queryParameters['username'] ?? '';
        String encryptedKey = uri.queryParameters['encryptedKey'] ?? '';
        // Now handle the callback with the username and encrypted key
        handleCallback(username, encryptedKey);
      }
    }, onError: (err) {
      // Handle error scenario
    });
  }


  Future<void> _openOnairosOverlay(BuildContext context, Function onResolved) async {
    print("Attempting to check credentials."); // Dart-side logging
    if(!await hasCredentials()){
      print("No credentials found, authenticating with Onairos."); // More logging
      bool cred = await hasCredentials();
      print("User has credentials verified : $cred");
      authenticateWithOnairos();
      return;
    }
    print("Credentials found, proceeding to get credentials."); // Confirming path taken
    // Recieve credientials via Access to Secure Overlay storage
    Map<String, String>? Credentials = await getCredentials();

    if (Credentials == null || Credentials['username'] == null || Credentials['pin'] == null) {
    // Check credentials and individual values for null
      // Show an error modal or pop the navigator if any credential is null
      showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        builder: (BuildContext context) => OnairosPromptModal(
          onClose: () => Navigator.of(context).pop(),
          logoPath: 'assets/images/onairos_logo.png', // Ensure you have the logo image in your assets
        ),
      );
      return; // Ensure we exit after handling the error case
    }

    String? OnairosUsername = Credentials['username']; 
    String? key = Credentials['key']; 

    try {
      showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        builder: (context) => OnairosOverlay(
          data: widget.requestData ?? {'Default': 'No data'},
          username: OnairosUsername!,
          modelKey: key!,
          onResolved: widget.onResolved,
        ),
      );
    } catch (e) {
      // Handle any other exceptions that might occur
      showErrorModal(context, 'An error occurred: $e');
      throw 'Could not overlay with ${widget.requestData}';
    }
  }

  void openOverlay(String username, String pin){
    if (username == null || pin == null) {
    // Check credentials and individual values for null
      // Show an error modal or pop the navigator if any credential is null
      showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        builder: (BuildContext context) => OnairosPromptModal(
          onClose: () => Navigator.of(context).pop(),
          logoPath: 'assets/images/onairos_logo.png', // Ensure you have the logo image in your assets
        ),
      );
      return; // Ensure we exit after handling the error case
    }
    try {
      showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        builder: (context) => OnairosOverlay(
          data: widget.requestData ?? {'Default': 'No data'},
          username: username,
          modelKey: pin,
          onResolved: widget.onResolved,
        ),
      );
    } catch (e) {
      // Handle any other exceptions that might occur
      showErrorModal(context, 'An error occurred: $e');
      throw 'Could not overlay with ${widget.requestData}';
    }
  }

  void showErrorModal(BuildContext context, String errorMessage) {
    if (mounted) {
      showDialog(
        context: context,
        builder: (context) {
          return AlertDialog(
            title: Text('Error'),
            content: Text(errorMessage),
            actions: [
              TextButton(
                onPressed: () => Navigator.of(context).pop(),
                child: Text('OK'),
              ),
            ],
          );
        },
      );
    }
  }

  // Check if credentials are already stored
  Future<bool> hasCredentials() async {
    print("Calling Swift to check credentials");
    return await _channel.invokeMethod('hasCredentials') ?? false;
  }

  Future<Map<String, String>?> getCredentials() async {
    final result = await _channel.invokeMethod<Map<dynamic, dynamic>>('getUserInfo');
    return result?.map((key, value) => MapEntry(key as String, value as String));
  }

  // Method to open Onairos App for authentication
  Future<void> authenticateWithOnairos() async {
    String name = "App Name";  // Ensure this is URL-safe or properly encoded
    int nonce = DateTime.now().millisecondsSinceEpoch;
    Uri onairosScheme = Uri(
      scheme: 'onairos',
      host: 'authenticate',
      queryParameters: {
        'nonce': nonce.toString(),
        'callback': '${widget.returnLink}://callback',
        'appName': name,
      }
    );
    print("Onairos button Deep Link URL launched: ${onairosScheme.toString()}");
    try {
      if (await canLaunchUrl(onairosScheme)) {
        await launchUrl(onairosScheme);
      } else {
        throw Exception('Could not launch Onairos app');
      }
    } catch (e) {
      showErrorModal(context, 'Failed to authenticate: $e');
    }
  }



  // Called by the platform-specific code to handle the callback
  Future<void> handleCallback(String username, String encryptedKey) async {
    await _channel.invokeMethod('storeCredentials', {'username': username, 'encryptedKey': encryptedKey});
    openOverlay(username, encryptedKey);
  }

  @override
  Widget build(BuildContext context) {
    // Rendere Onairs Button
    return OnairosButton(
      requestData: widget.requestData,
      returnLink: widget.returnLink,
      openOnairosOverlay: _openOnairosOverlay,
      onResolved: (apiUrl, accessToken) {
        widget.onResolved;// App2 can define how to handle the apiUrl here
      },
    );
  }
}

// Rendered Button which user clicks to open overlay and initiate data request
class OnairosButton extends StatelessWidget {
  final Map<String, Map<String, String>> requestData;
  final String returnLink;
  final Function onResolved;
  final Future<void> Function(BuildContext context, Function onResolved) openOnairosOverlay;

  const OnairosButton({
    Key? key,
    required this.requestData,
    required this.onResolved,
    required this.returnLink,
    required this.openOnairosOverlay,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      // onPressed: () => openOnairosOverlay(context, () => onResolved(apiUrl, accessToken)),
      onPressed: () => openOnairosOverlay(context, onResolved),
      child: Image.asset('images/onairos_logo.png'),
      style: ElevatedButton.styleFrom(
        foregroundColor: Colors.grey,
        backgroundColor: Colors.white,
        shape: CircleBorder(),
        padding: EdgeInsets.all(10),
      ),
    );
  }
}